Package: ParamHelpers
Title: Helpers for Parameters in Black-Box Optimization, Tuning and
        Machine Learning
Version: 1.14.2
Authors@R: 
    c(person(given = "Bernd",
             family = "Bischl",
             role = "aut",
             email = "bernd_bischl@gmx.net",
             comment = c(ORCID = "0000-0001-6002-6980")),
      person(given = "Michel",
             family = "Lang",
             role = "aut",
             email = "michellang@gmail.com",
             comment = c(ORCID = "0000-0001-9754-0393")),
      person(given = "Jakob",
             family = "Richter",
             role = "aut",
             email = "code@jakob-r.de",
             comment = c(ORCID = "0000-0003-4481-5554")),
      person(given = "Jakob",
             family = "Bossek",
             role = "aut",
             email = "j.bossek@gmail.com"),
      person(given = "Daniel",
             family = "Horn",
             role = "aut",
             email = "daniel.horn@tu-dortmund.de"),
      person(given = "Karin",
             family = "Schork",
             role = "ctb",
             email = "karin.schork@tu-dortmund.de"),
      person(given = "Pascal",
             family = "Kerschke",
             role = "aut",
             email = "kerschke@uni-muenster.de"),
      person(given = "Martin",
             family = "Binder",
             role = c("ctb", "cre"),
             email = "mlr.developer@mb706.com"))
Description: Functions for parameter descriptions and operations in
    black-box optimization, tuning and machine learning. Parameters can be
    described (type, constraints, defaults, etc.), combined to parameter
    sets and can in general be programmed on. A useful OptPath object
    (archive) to log function evaluations is also provided.
License: BSD_2_clause + file LICENSE
URL: https://paramhelpers.mlr-org.com,
        https://github.com/mlr-org/ParamHelpers
BugReports: https://github.com/mlr-org/ParamHelpers/issues
Imports: backports, BBmisc (>= 1.10), checkmate (>= 1.8.2), fastmatch,
        methods
Suggests: akima, covr, eaf, emoa, GGally, ggplot2, grid, gridExtra,
        irace (>= 2.1), lhs, plyr, reshape2, testthat
ByteCompile: yes
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-01-09 15:17:24 UTC; user
Author: Bernd Bischl [aut] (<https://orcid.org/0000-0001-6002-6980>),
  Michel Lang [aut] (<https://orcid.org/0000-0001-9754-0393>),
  Jakob Richter [aut] (<https://orcid.org/0000-0003-4481-5554>),
  Jakob Bossek [aut],
  Daniel Horn [aut],
  Karin Schork [ctb],
  Pascal Kerschke [aut],
  Martin Binder [ctb, cre]
Maintainer: Martin Binder <mlr.developer@mb706.com>
Repository: CRAN
Date/Publication: 2025-01-09 22:50:02 UTC
