# Generated by roxygen2: do not edit by hand

S3method(addOptPathEl,OptPathDF)
S3method(as.data.frame,OptPathDF)
S3method(c,ParamSet)
S3method(evaluateParamExpressions,Param)
S3method(evaluateParamExpressions,ParamSet)
S3method(evaluateParamExpressions,list)
S3method(getBounds,Param)
S3method(getBounds,ParamSet)
S3method(getBounds,list)
S3method(getDefaults,Param)
S3method(getDefaults,ParamSet)
S3method(getDefaults,list)
S3method(getOptPathCol,OptPathDF)
S3method(getOptPathCols,OptPathDF)
S3method(getOptPathDOB,OptPathDF)
S3method(getOptPathEOL,OptPathDF)
S3method(getOptPathEl,OptPathDF)
S3method(getOptPathErrorMessages,OptPathDF)
S3method(getOptPathExecTimes,OptPathDF)
S3method(getOptPathLength,OptPathDF)
S3method(getOptPathX,OptPathDF)
S3method(getOptPathY,OptPathDF)
S3method(getParamIds,Param)
S3method(getParamIds,ParamSet)
S3method(getParamLengths,Param)
S3method(getParamLengths,ParamSet)
S3method(getRequiredParamNames,Param)
S3method(getRequiredParamNames,ParamSet)
S3method(getValues,Param)
S3method(getValues,ParamSet)
S3method(getValues,list)
S3method(hasExpression,LearnerParam)
S3method(hasExpression,LearnerParamSet)
S3method(hasExpression,Param)
S3method(hasExpression,ParamSet)
S3method(hasExpression,list)
S3method(hasFiniteBoxConstraints,Param)
S3method(hasFiniteBoxConstraints,ParamSet)
S3method(hasRequires,Param)
S3method(hasRequires,ParamSet)
S3method(hasTrafo,Param)
S3method(hasTrafo,ParamSet)
S3method(isCharacter,Param)
S3method(isCharacter,ParamSet)
S3method(isDiscrete,Param)
S3method(isDiscrete,ParamSet)
S3method(isEmpty,ParamSet)
S3method(isFeasible,LearnerParam)
S3method(isFeasible,Param)
S3method(isFeasible,ParamSet)
S3method(isInteger,Param)
S3method(isInteger,ParamSet)
S3method(isLogical,Param)
S3method(isLogical,ParamSet)
S3method(isNumeric,Param)
S3method(isNumeric,ParamSet)
S3method(isVector,Param)
S3method(isVector,ParamSet)
S3method(paramValueToString,Param)
S3method(paramValueToString,ParamSet)
S3method(print,LearnerParam)
S3method(print,OptPath)
S3method(print,Param)
S3method(print,ParamSet)
S3method(sampleValue,Param)
S3method(sampleValue,ParamSet)
S3method(setValueCNames,Param)
S3method(setValueCNames,ParamSet)
export(addOptPathEl)
export(checkParamSet)
export(convertParamSetToIrace)
export(dfRowToList)
export(dfRowsToList)
export(discreteNameToValue)
export(discreteValueToName)
export(dropParams)
export(evaluateParamExpressions)
export(filterParams)
export(filterParamsDiscrete)
export(filterParamsNumeric)
export(generateDesign)
export(generateDesignOfDefaults)
export(generateGridDesign)
export(generateRandomDesign)
export(getDefaults)
export(getLower)
export(getOptPathBestIndex)
export(getOptPathCol)
export(getOptPathCols)
export(getOptPathDOB)
export(getOptPathEOL)
export(getOptPathEl)
export(getOptPathErrorMessages)
export(getOptPathExecTimes)
export(getOptPathLength)
export(getOptPathParetoFront)
export(getOptPathX)
export(getOptPathY)
export(getParamIds)
export(getParamLengths)
export(getParamNr)
export(getParamSet)
export(getParamTypeCounts)
export(getParamTypes)
export(getRequirements)
export(getTypeStringsAll)
export(getTypeStringsCharacter)
export(getTypeStringsDiscrete)
export(getTypeStringsInteger)
export(getTypeStringsLogical)
export(getTypeStringsNumeric)
export(getTypeStringsNumericStrict)
export(getUpper)
export(getValues)
export(hasCharacter)
export(hasDiscrete)
export(hasExpression)
export(hasFiniteBoxConstraints)
export(hasForbidden)
export(hasInteger)
export(hasLogical)
export(hasNumeric)
export(hasRequires)
export(hasTrafo)
export(isCharacter)
export(isCharacterTypeString)
export(isDiscrete)
export(isDiscreteTypeString)
export(isEmpty)
export(isFeasible)
export(isForbidden)
export(isInteger)
export(isIntegerTypeString)
export(isLogical)
export(isLogicalTypeString)
export(isNumeric)
export(isNumericTypeString)
export(isRequiresOk)
export(isSpecialValue)
export(isVector)
export(isVectorTypeString)
export(makeCharacterParam)
export(makeCharacterVectorParam)
export(makeDiscreteLearnerParam)
export(makeDiscreteParam)
export(makeDiscreteVectorLearnerParam)
export(makeDiscreteVectorParam)
export(makeFunctionLearnerParam)
export(makeFunctionParam)
export(makeIntegerLearnerParam)
export(makeIntegerParam)
export(makeIntegerVectorLearnerParam)
export(makeIntegerVectorParam)
export(makeLogicalLearnerParam)
export(makeLogicalParam)
export(makeLogicalVectorLearnerParam)
export(makeLogicalVectorParam)
export(makeNumericLearnerParam)
export(makeNumericParam)
export(makeNumericParamSet)
export(makeNumericVectorLearnerParam)
export(makeNumericVectorParam)
export(makeOptPathDF)
export(makeParamSet)
export(makeUntypedLearnerParam)
export(makeUntypedParam)
export(paramValueToString)
export(plotEAF)
export(plotOptPath)
export(plotYTraces)
export(removeMissingValues)
export(renderOptPathPlot)
export(repairPoint)
export(sampleValue)
export(sampleValues)
export(setOptPathElDOB)
export(setOptPathElEOL)
export(setValueCNames)
export(trafoOptPath)
export(trafoValue)
export(updateParVals)
import(BBmisc, except = isFALSE)
import(checkmate)
import(methods)
import(stats)
importFrom(fastmatch,"%fin%")
importFrom(fastmatch,fmatch)
useDynLib(ParamHelpers,c_dfRowsToList)
