% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OptPath_setter.R
\name{setOptPathElEOL}
\alias{setOptPathElEOL}
\title{Set the end of life dates of parameter values, in-place.}
\usage{
setOptPathElEOL(op, index, eol)
}
\arguments{
\item{op}{\link{OptPath}\cr
Optimization path.}

\item{index}{\link{integer}\cr
Vector of indices of elements.}

\item{eol}{\link{integer} \cr
EOL dates, single value or same length of \code{index}.}
}
\value{
Nothing.
}
\description{
Set the end of life dates of parameter values, in-place.
}
\seealso{
Other optpath: 
\code{\link{OptPath}},
\code{\link{addOptPathEl}()},
\code{\link{getOptPathBestIndex}()},
\code{\link{getOptPathCol}()},
\code{\link{getOptPathCols}()},
\code{\link{getOptPathDOB}()},
\code{\link{getOptPathEOL}()},
\code{\link{getOptPathEl}()},
\code{\link{getOptPathErrorMessages}()},
\code{\link{getOptPathExecTimes}()},
\code{\link{getOptPathLength}()},
\code{\link{getOptPathParetoFront}()},
\code{\link{getOptPathX}()},
\code{\link{getOptPathY}()},
\code{\link{setOptPathElDOB}()}
}
\concept{optpath}
